/********************************************************************
*                   LIBRERIA USO USART
* 
*   - Funciones para transmision y recepcion de caracteres
 *******************************************************************/

#include "usart.h"
#include <stdbool.h>
#include <xc.h>


/******************************************************************************
*                       CONFIGURACION DEL PUERTO
* 
*   - Realiza la inicializacin del puerto segn los defines de usart.h
 *****************************************************************************/

void USART_Initialize()
{
    
UART_TRISTx = 1;        //Configura TX como entrada
UART_TRISRx = 1;        //Configura RX como entrada - Inicialmente se configura como entrada
    
UART_ENABLE = 1;        //Habilita la comunicacion serie
UART_RECEP_ENABLE = 1;  //Habilita la recepcion serie continua
UART_TRANS_ENABLE = 1;  //Habilita la transmision serie
         
SPBRG = ((int)(_XTAL_FREQ/(MULTIPLIER * BAUD) -1)); //Inicializa el registro segn los 
                                                    //parametros de usart.h


if (HIGH_SPEED == 1)
{
    RCSTA = (RCSTA_DEFAULT);
    TXSTA = (0x4 | TXSTA_DEFAULT);
}
else
{
    RCSTA = (RCSTA_DEFAULT);
    TXSTA = (0x0 | TXSTA_DEFAULT);
}
    

}

/******************************************************************************
*                       ESCRIBIR CARACTER
* 
*   - Funcion que transmite un caracter cuando el buffer de salida esta libre
*   - La funcin "printf" llama por defecto a la funcion putch  
 *****************************************************************************/
void USART_putcUSART(char c)
{
while(!TRMT);       // Espera a que el buffer de salida este libre
TXREG1 = c;         // Envia el caracter
}

void putch (char c)     //A esta funcion llama printf, deber ser nombrada asi
{
while(!TRMT);       // Espera a que el buffer de salida este libre
TXREG1 = c;         // Envia el caracter
}


/******************************************************************************
*                       ESTADO BUFFER SALIDA
* 
*   -  Funcin que devuelve el estado del buffer de salida
*      TRUE: Libre,  FALSE: Ocupado 
 *****************************************************************************/
bool UART_TX_Empty()
{
  return TRMT;      //Bit de estado del buffer de transmision
}


/******************************************************************************
*                       ESCRIBIR FRASE
* 
*   - Funcion encargada de escribir caracteres contenidos en un array usando
*     la funcin USART_putcUSART 
 *****************************************************************************/
void UART_Write_Text(char *text)
{
  int i;
  for(i=0;text[i]!='\0';i++)
    USART_putcUSART(text[i]);
}



/******************************************************************************
*                       RECIBIDO ALGO
* 
*   - Funcion que devuelve el estado del buffer de entrada 
*     TRUE: Libre,  FALSE: Ocupado 
 *****************************************************************************/
bool UART_Data_Ready()
{
  return RCIF;      //Bandera que indica que hemos recibido algo
}


/******************************************************************************
*                       LEER CARACTER
* 
*   - Funcion que devuelve el caracter leido del buffer de entrada 
 *****************************************************************************/
unsigned char USART_getcUSART ()
{
	char  c = 0;
        if (RCSTA1bits.OERR == 1) 
            { 
            RCSTA1bits.OERR = 0;      // clear overrun if it occurs
            RCSTA1bits.CREN = 0; 
            RCSTA1bits.CREN = 1; 
            }

	c = RCREG1;
    return c;
}


/******************************************************************************
*                       LEER CARACTER O ESPERAR CARACTER
* 
*   - Funcion que devuelve el caracter leido del buffer de entrada 
*   - Esta funcion se diferencia de la anterior que espera si no hay nada en
*     el buffer. Cuando lo reciba lo devuelve.  
 *****************************************************************************/
unsigned char USART_getcUSART2 ()
{
	char  c = 0;
    while (!PIR1bits.RCIF);
        if (RCSTA1bits.OERR == 1) 
            { 
            RCSTA1bits.OERR = 0;      // clear overrun if it occurs
            RCSTA1bits.CREN = 0; 
            RCSTA1bits.CREN = 1; 
            }

	c = RCREG1;
    return c;
}


/******************************************************************************
*                       LEER GRUPO DE CARACTERES
* 
*   - Funcion que guarda en un array el contenido del buffer de entrada
*   - Usa la funcion que espera por caracter en buffer y no termina hasta que 
*     haya completado el array. 
 *****************************************************************************/
void USART_ReadString(char *Output, unsigned int length)
{
  unsigned int i;
  for(int i=0;i<length;i++)
  {
  Output[i] = USART_getcUSART2();
  }
}

/******************************************************************************
*                       LEER GRUPO DE CARACTERES
* 
*   - Funcion que guarda en un array el contenido del buffer de entrada
*   - Usa la funcion que espera por caracter en buffer y no termina hasta que 
*     encuentre el caracter fin de linea '\n'
 *****************************************************************************/
void USART_ReadString2(char *Output)
{
    unsigned int i = 0;
    char caracter = 0;
    while(caracter != '\n')
    {
    caracter = USART_getcUSART2();
    Output[i] = caracter; 
    i++;
    }
}








